@@ -335,6 +335,8 @@ typedef enum {
  *
  * All pending timeouts or periodic notifies are converted into
  * an entry.
+ * Note that GstClockEntry should be treated as an opaque structure. It must
+ * not be extended or allocated using a custom allocator.
  */
 struct _GstClockEntry {
   gint			refcount;
@@ -346,6 +348,7 @@ struct _GstClockEntry {
   GstClockReturn	 status;
   GstClockCallback	 func;
   gpointer		 user_data;
+  GDestroyNotify	 destroy_data;
 };
 
 /**
@@ -544,6 +547,10 @@ GstClockReturn		gst_clock_id_wait		(GstClockID id,
 GstClockReturn		gst_clock_id_wait_async		(GstClockID id,
 							 GstClockCallback func,
 							 gpointer user_data);
+GstClockReturn		gst_clock_id_wait_async_full	(GstClockID id,
+							 GstClockCallback func,
+							 gpointer user_data,
+							 GDestroyNotify destroy_data);
 void			gst_clock_id_unschedule		(GstClockID id);
 
 
