@@ -40,12 +40,12 @@
  * <title>Using an element factory</title>
  * <programlisting language="c">
  *   #include &lt;gst/gst.h&gt;
- *   
+ *
  *   GstElement *src;
  *   GstElementFactory *srcfactory;
- *   
+ *
  *   gst_init (&amp;argc, &amp;argv);
- *   
+ *
  *   srcfactory = gst_element_factory_find ("filesrc");
  *   g_return_if_fail (srcfactory != NULL);
  *   src = gst_element_factory_create (srcfactory, "src");
@@ -60,6 +60,7 @@
 #include "gst_private.h"
 
 #include "gstelement.h"
+#include "gstelementdetails.h"
 #include "gstinfo.h"
 #include "gsturi.h"
 #include "gstregistry.h"
@@ -69,10 +70,7 @@
 GST_DEBUG_CATEGORY_STATIC (element_factory_debug);
 #define GST_CAT_DEFAULT element_factory_debug
 
-static void gst_element_factory_class_init (GstElementFactoryClass * klass);
-static void gst_element_factory_init (GstElementFactory * factory);
 static void gst_element_factory_finalize (GObject * object);
-void __gst_element_details_clear (GstElementDetails * dp);
 static void gst_element_factory_cleanup (GstElementFactory * factory);
 
 static GstPluginFeatureClass *parent_class = NULL;
@@ -99,7 +97,7 @@ gst_element_factory_class_init (GstElementFactoryClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_element_factory_finalize);
+  gobject_class->finalize = gst_element_factory_finalize;
 }
 
 static void
@@ -150,45 +148,6 @@ gst_element_factory_find (const gchar * name)
   return NULL;
 }
 
-void
-__gst_element_details_clear (GstElementDetails * dp)
-{
-  g_free (dp->longname);
-  g_free (dp->klass);
-  g_free (dp->description);
-  g_free (dp->author);
-  memset (dp, 0, sizeof (GstElementDetails));
-}
-
-#define VALIDATE_SET(__dest, __src, __entry)                            \
-G_STMT_START {                                                          \
-  if (g_utf8_validate (__src->__entry, -1, NULL)) {                     \
-    __dest->__entry = g_strdup (__src->__entry);                        \
-  } else {                                                              \
-    g_warning ("Invalid UTF-8 in " G_STRINGIFY (__entry) ": %s",        \
-        __src->__entry);                                                \
-    __dest->__entry = g_strdup ("[ERROR: invalid UTF-8]");              \
-  }                                                                     \
-} G_STMT_END
-
-void
-__gst_element_details_set (GstElementDetails * dest,
-    const GstElementDetails * src)
-{
-  VALIDATE_SET (dest, src, longname);
-  VALIDATE_SET (dest, src, klass);
-  VALIDATE_SET (dest, src, description);
-  VALIDATE_SET (dest, src, author);
-}
-
-void
-__gst_element_details_copy (GstElementDetails * dest,
-    const GstElementDetails * src)
-{
-  __gst_element_details_clear (dest);
-  __gst_element_details_set (dest, src);
-}
-
 static void
 gst_element_factory_cleanup (GstElementFactory * factory)
 {
@@ -218,7 +177,7 @@ gst_element_factory_cleanup (GstElementFactory * factory)
       g_ptr_array_free (caps->structs, TRUE);
       caps->refcount = 0;
     }
-    g_free (templ);
+    g_slice_free (GstStaticPadTemplate, templ);
   }
   g_list_free (factory->staticpadtemplates);
   factory->staticpadtemplates = NULL;
@@ -265,8 +224,8 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   registry = gst_registry_get_default ();
 
   /* check if feature already exists, if it exists there is no need to update it
-   * when the registry is getting updated, outdated plugins and all there
-   * feature are removed and readded.
+   * when the registry is getting updated, outdated plugins and all their
+   * features are removed and readded.
    */
   existing_feature = gst_registry_lookup_feature (registry, name);
   if (existing_feature) {
@@ -281,11 +240,14 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
   }
 
   factory =
-      GST_ELEMENT_FACTORY_CAST (g_object_new (GST_TYPE_ELEMENT_FACTORY, NULL));
+      GST_ELEMENT_FACTORY_CAST (g_object_newv (GST_TYPE_ELEMENT_FACTORY, 0,
+          NULL));
   gst_plugin_feature_set_name (GST_PLUGIN_FEATURE_CAST (factory), name);
   GST_LOG_OBJECT (factory, "Created new elementfactory for type %s",
       g_type_name (type));
 
+  /* provide info needed during class structure setup */
+  g_type_set_qdata (type, _gst_elementclass_factory, factory);
   klass = GST_ELEMENT_CLASS (g_type_class_ref (type));
   if ((klass->details.longname == NULL) ||
       (klass->details.klass == NULL) || (klass->details.author == NULL))
@@ -297,16 +259,16 @@ gst_element_register (GstPlugin * plugin, const gchar * name, guint rank,
     GstPadTemplate *templ = item->data;
     GstStaticPadTemplate *newt;
 
-    newt = g_new0 (GstStaticPadTemplate, 1);
+    newt = g_slice_new (GstStaticPadTemplate);
     newt->name_template = g_intern_string (templ->name_template);
     newt->direction = templ->direction;
     newt->presence = templ->presence;
+    newt->static_caps.caps.refcount = 0;
     newt->static_caps.string = gst_caps_to_string (templ->caps);
     factory->staticpadtemplates =
         g_list_append (factory->staticpadtemplates, newt);
   }
   factory->numpadtemplates = klass->numpadtemplates;
-  g_type_set_qdata (type, _gst_elementclass_factory, factory);
 
   /* special stuff for URI handling */
   if (g_type_is_a (type, GST_TYPE_URI_HANDLER)) {
@@ -403,13 +365,19 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
   if (factory->type == 0)
     goto no_type;
 
-  /* create an instance of the element, cast so we don't assert on NULL */
-  element = GST_ELEMENT_CAST (g_object_new (factory->type, NULL));
+  /* create an instance of the element, cast so we don't assert on NULL
+   * also set name as early as we can
+   */
+  if (name)
+    element =
+        GST_ELEMENT_CAST (g_object_new (factory->type, "name", name, NULL));
+  else
+    element = GST_ELEMENT_CAST (g_object_newv (factory->type, 0, NULL));
   if (G_UNLIKELY (element == NULL))
     goto no_element;
 
   /* fill in the pointer to the factory in the element class. The
-   * class will not be unreffed currently. 
+   * class will not be unreffed currently.
    * Be thread safe as there might be 2 threads creating the first instance of
    * an element at the same moment
    */
@@ -418,9 +386,6 @@ gst_element_factory_create (GstElementFactory * factory, const gchar * name)
           (gpointer) & oclass->elementfactory, NULL, factory))
     gst_object_unref (factory);
 
-  if (name)
-    gst_object_set_name (GST_OBJECT_CAST (element), name);
-
   GST_DEBUG ("created element \"%s\"", GST_PLUGIN_FEATURE_NAME (factory));
 
   return element;
@@ -449,7 +414,7 @@ no_element:
 /**
  * gst_element_factory_make:
  * @factoryname: a named factory to instantiate
- * @name: name of new element
+ * @name: (allow-none): name of new element
  *
  * Create a new element of the type defined by the given element factory.
  * If name is NULL, then the element will receive a guaranteed unique name,
