@@ -104,7 +104,7 @@ enum
 {
   PARENT_SET,
   PARENT_UNSET,
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   OBJECT_SAVED,
 #endif
   DEEP_NOTIFY,
@@ -134,7 +134,7 @@ typedef struct _GstSignalObjectClass GstSignalObjectClass;
 
 static GType gst_signal_object_get_type (void);
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 static guint gst_signal_object_signals[SO_LAST_SIGNAL] = { 0 };
 #endif
 
@@ -150,9 +150,22 @@ static void gst_object_finalize (GObject * object);
 
 static gboolean gst_object_set_name_default (GstObject * object);
 
-#ifndef GST_DISABLE_LOADSAVE
+#ifdef GST_DISABLE_DEPRECATED
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
+#undef GstXmlNodePtr
+#define GstXmlNodePtr xmlNodePtr
+#include <libxml/parser.h>
+GstXmlNodePtr gst_object_save_thyself (GstObject * object,
+    GstXmlNodePtr parent);
+void gst_object_restore_thyself (GstObject * object, GstXmlNodePtr parent);
+void gst_class_signal_emit_by_name (GstObject * object, const gchar * name,
+    GstXmlNodePtr self);
+#endif
+#endif
+
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 static void gst_object_real_restore_thyself (GstObject * object,
-    xmlNodePtr self);
+    GstXmlNodePtr self);
 #endif
 
 static GObjectClass *parent_class = NULL;
@@ -203,7 +216,7 @@ gst_object_class_init (GstObjectClass * klass)
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstObjectClass, parent_unset), NULL,
       NULL, g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_OBJECT);
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   /**
    * GstObject::object-saved:
    * @gstobject: a #GstObject
@@ -220,7 +233,9 @@ gst_object_class_init (GstObjectClass * klass)
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstObjectClass, object_saved), NULL,
       NULL, g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
 
-  klass->restore_thyself = gst_object_real_restore_thyself;
+  klass->restore_thyself =
+      ((void (*)(GstObject * object,
+              gpointer self)) *gst_object_real_restore_thyself);
 #endif
 
   /**
@@ -261,7 +276,7 @@ gst_object_init (GstObject * object)
   object->lock = g_mutex_new ();
   object->parent = NULL;
   object->name = NULL;
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "%p new", object);
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "%p new", object);
 
 #ifndef GST_DISABLE_TRACE
   gst_alloc_trace_new (_gst_object_trace, object);
@@ -291,8 +306,7 @@ gst_object_ref (gpointer object)
   g_return_val_if_fail (object != NULL, NULL);
 
 #ifdef DEBUG_REFCOUNT
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "%p ref %d->%d",
-      object,
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "%p ref %d->%d", object,
       ((GObject *) object)->ref_count, ((GObject *) object)->ref_count + 1);
 #endif
   g_object_ref (object);
@@ -318,8 +332,7 @@ gst_object_unref (gpointer object)
   g_return_if_fail (((GObject *) object)->ref_count > 0);
 
 #ifdef DEBUG_REFCOUNT
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "%p unref %d->%d",
-      object,
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "%p unref %d->%d", object,
       ((GObject *) object)->ref_count, ((GObject *) object)->ref_count - 1);
 #endif
   g_object_unref (object);
@@ -349,7 +362,8 @@ gst_object_ref_sink (gpointer object)
 
   GST_OBJECT_LOCK (object);
   if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
-    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unsetting floating flag");
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object,
+        "unsetting floating flag");
     GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
     GST_OBJECT_UNLOCK (object);
   } else {
@@ -380,11 +394,11 @@ gst_object_sink (gpointer object)
 {
   g_return_if_fail (GST_IS_OBJECT (object));
 
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "sink");
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "sink");
 
   GST_OBJECT_LOCK (object);
   if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
-    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "clear floating flag");
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "clear floating flag");
     GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
     GST_OBJECT_UNLOCK (object);
     gst_object_unref (object);
@@ -415,7 +429,7 @@ gst_object_replace (GstObject ** oldobj, GstObject * newobj)
   g_return_if_fail (newobj == NULL || GST_IS_OBJECT (newobj));
 
 #ifdef DEBUG_REFCOUNT
-  GST_CAT_LOG (GST_CAT_REFCOUNTING, "replace %p %s (%d) with %p %s (%d)",
+  GST_CAT_TRACE (GST_CAT_REFCOUNTING, "replace %p %s (%d) with %p %s (%d)",
       *oldobj, *oldobj ? GST_STR_NULL (GST_OBJECT_NAME (*oldobj)) : "(NONE)",
       *oldobj ? G_OBJECT (*oldobj)->ref_count : 0,
       newobj, newobj ? GST_STR_NULL (GST_OBJECT_NAME (newobj)) : "(NONE)",
@@ -439,7 +453,7 @@ gst_object_dispose (GObject * object)
 {
   GstObject *parent;
 
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
 
   GST_OBJECT_LOCK (object);
   if ((parent = GST_OBJECT_PARENT (object)))
@@ -471,7 +485,7 @@ gst_object_finalize (GObject * object)
 {
   GstObject *gstobject = GST_OBJECT_CAST (object);
 
-  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "finalize");
+  GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "finalize");
 
   g_signal_handlers_destroy (object);
 
@@ -727,7 +741,14 @@ gst_object_get_name (GstObject * object)
  * retains ownership of the name prefix it sent.
  *
  * MT safe.  This function grabs and releases @object's LOCK.
+ *
+ * Deprecated: deprecated because the name prefix has never actually been used
+ *     for anything.
  */
+#ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
+void gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix);
+#endif
 void
 gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix)
 {
@@ -738,6 +759,7 @@ gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix)
   object->name_prefix = g_strdup (name_prefix); /* NULL gives NULL */
   GST_OBJECT_UNLOCK (object);
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 /**
  * gst_object_get_name_prefix:
@@ -751,7 +773,14 @@ gst_object_set_name_prefix (GstObject * object, const gchar * name_prefix)
  * Returns: the name prefix of @object. g_free() after usage.
  *
  * MT safe. This function grabs and releases @object's LOCK.
+ *
+ * Deprecated: deprecated because the name prefix has never actually been used
+ *     for anything.
  */
+#ifndef GST_REMOVE_DEPRECATED
+#ifdef GST_DISABLE_DEPRECATED
+gchar *gst_object_get_name_prefix (GstObject * object);
+#endif
 gchar *
 gst_object_get_name_prefix (GstObject * object)
 {
@@ -765,6 +794,7 @@ gst_object_get_name_prefix (GstObject * object)
 
   return result;
 }
+#endif /* GST_REMOVE_DEPRECATED */
 
 /**
  * gst_object_set_parent:
@@ -801,7 +831,8 @@ gst_object_set_parent (GstObject * object, GstObject * parent)
    * in the floating case. */
   object->parent = parent;
   if (G_LIKELY (GST_OBJECT_IS_FLOATING (object))) {
-    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unsetting floating flag");
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object,
+        "unsetting floating flag");
     GST_OBJECT_FLAG_UNSET (object, GST_OBJECT_FLOATING);
     GST_OBJECT_UNLOCK (object);
   } else {
@@ -871,7 +902,7 @@ gst_object_unparent (GstObject * object)
   parent = object->parent;
 
   if (G_LIKELY (parent != NULL)) {
-    GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unparent");
+    GST_CAT_TRACE_OBJECT (GST_CAT_REFCOUNTING, object, "unparent");
     object->parent = NULL;
     GST_OBJECT_UNLOCK (object);
 
@@ -960,7 +991,7 @@ gst_object_check_uniqueness (GList * list, const gchar * name)
 }
 
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 /**
  * gst_object_save_thyself:
  * @object: a #GstObject to save
@@ -970,8 +1001,8 @@ gst_object_check_uniqueness (GList * list, const gchar * name)
  *
  * Returns: the new xmlNodePtr with the saved object
  */
-xmlNodePtr
-gst_object_save_thyself (GstObject * object, xmlNodePtr parent)
+GstXmlNodePtr
+gst_object_save_thyself (GstObject * object, GstXmlNodePtr parent)
 {
   GstObjectClass *oclass;
 
@@ -996,7 +1027,7 @@ gst_object_save_thyself (GstObject * object, xmlNodePtr parent)
  * Restores @object with the data from the parent XML node.
  */
 void
-gst_object_restore_thyself (GstObject * object, xmlNodePtr self)
+gst_object_restore_thyself (GstObject * object, GstXmlNodePtr self)
 {
   GstObjectClass *oclass;
 
@@ -1010,7 +1041,7 @@ gst_object_restore_thyself (GstObject * object, xmlNodePtr self)
 }
 
 static void
-gst_object_real_restore_thyself (GstObject * object, xmlNodePtr self)
+gst_object_real_restore_thyself (GstObject * object, GstXmlNodePtr self)
 {
   g_return_if_fail (GST_IS_OBJECT (object));
   g_return_if_fail (self != NULL);
@@ -1150,9 +1181,9 @@ struct _GstSignalObjectClass
   GObjectClass parent_class;
 
   /* signals */
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   void (*object_loaded) (GstSignalObject * object, GstObject * new,
-      xmlNodePtr self);
+      GstXmlNodePtr self);
 #endif
 };
 
@@ -1163,7 +1194,7 @@ gst_signal_object_class_init (GstSignalObjectClass * klass)
 {
   parent_class = g_type_class_peek_parent (klass);
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   gst_signal_object_signals[SO_OBJECT_LOADED] =
       g_signal_new ("object-loaded", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstSignalObjectClass, object_loaded),
@@ -1198,7 +1229,7 @@ gst_class_signal_connect (GstObjectClass * klass,
       func_data);
 }
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 /**
  * gst_class_signal_emit_by_name:
  * @object: a #GstObject that emits the signal
@@ -1209,7 +1240,7 @@ gst_class_signal_connect (GstObjectClass * klass,
  */
 void
 gst_class_signal_emit_by_name (GstObject * object,
-    const gchar * name, xmlNodePtr self)
+    const gchar * name, GstXmlNodePtr self)
 {
   GstObjectClass *oclass;
 
