@@ -35,12 +35,11 @@
 #include "gstregistry.h"
 #include "gstinfo.h"
 
+#include <stdio.h>
 #include <string.h>
 
 #define GST_CAT_DEFAULT GST_CAT_PLUGIN_LOADING
 
-static void gst_plugin_feature_class_init (GstPluginFeatureClass * klass);
-static void gst_plugin_feature_init (GstPluginFeature * feature);
 static void gst_plugin_feature_finalize (GObject * object);
 
 /* static guint gst_plugin_feature_signals[LAST_SIGNAL] = { 0 }; */
@@ -53,14 +52,13 @@ gst_plugin_feature_class_init (GstPluginFeatureClass * klass)
 {
   parent_class = g_type_class_peek_parent (klass);
 
-  G_OBJECT_CLASS (klass)->finalize =
-      GST_DEBUG_FUNCPTR (gst_plugin_feature_finalize);
+  G_OBJECT_CLASS (klass)->finalize = gst_plugin_feature_finalize;
 }
 
 static void
 gst_plugin_feature_init (GstPluginFeature * feature)
 {
-
+  /* do nothing, needed because of G_DEFINE_TYPE */
 }
 
 static void
@@ -71,7 +69,6 @@ gst_plugin_feature_finalize (GObject * object)
   GST_DEBUG ("finalizing feature %p: '%s'", feature,
       GST_PLUGIN_FEATURE_NAME (feature));
   g_free (feature->name);
-  g_free (feature->plugin_name);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -84,14 +81,14 @@ gst_plugin_feature_finalize (GObject * object)
  * unaffected; use the return value instead.
  *
  * Normally this function is used like this:
- *
- * <programlisting>
+ * |[
  * GstPluginFeature *loaded_feature;
+ * 
  * loaded_feature = gst_plugin_feature_load (feature);
  * // presumably, we're no longer interested in the potentially-unloaded feature
  * gst_object_unref (feature);
  * feature = loaded_feature;
- * </programlisting>
+ * ]|
  *
  * Returns: A reference to the loaded feature, or NULL on error.
  */
@@ -162,6 +159,8 @@ gboolean
 gst_plugin_feature_type_name_filter (GstPluginFeature * feature,
     GstTypeNameData * data)
 {
+  g_return_val_if_fail (GST_IS_PLUGIN_FEATURE (feature), FALSE);
+
   return ((data->type == 0 || data->type == G_OBJECT_TYPE (feature)) &&
       (data->name == NULL
           || !strcmp (data->name, GST_PLUGIN_FEATURE_NAME (feature))));
@@ -188,7 +187,7 @@ gst_plugin_feature_set_name (GstPluginFeature * feature, const gchar * name)
   } else {
     feature->name = g_strdup (name);
   }
-  gst_object_set_name (GST_OBJECT (feature), feature->name);
+  gst_object_set_name (GST_OBJECT_CAST (feature), feature->name);
 }
 
 /**
@@ -259,6 +258,43 @@ gst_plugin_feature_list_free (GList * list)
   g_list_free (list);
 }
 
+/**
+ * gst_plugin_feature_list_copy:
+ * @list: list of #GstPluginFeature
+ *
+ * Copies the list of features. Caller should call @gst_plugin_feature_list_free
+ * when done with the list.
+ *
+ * Returns: a copy of @list, with each feature's reference count incremented.
+ *
+ * Since: 0.10.26
+ */
+GList *
+gst_plugin_feature_list_copy (GList * list)
+{
+  GList *new_list = NULL;
+
+  if (G_LIKELY (list)) {
+    GList *last;
+
+    new_list = g_list_alloc ();
+    new_list->data = g_object_ref ((GObject *) list->data);
+    new_list->prev = NULL;
+    last = new_list;
+    list = list->next;
+    while (list) {
+      last->next = g_list_alloc ();
+      last->next->prev = last;
+      last = last->next;
+      last->data = g_object_ref ((GObject *) list->data);
+      list = list->next;
+    }
+    last->next = NULL;
+  }
+
+  return new_list;
+}
+
 /**
  * gst_plugin_feature_check_version:
  * @feature: a feature
@@ -291,12 +327,16 @@ gst_plugin_feature_check_version (GstPluginFeature * feature,
 
   if (plugin) {
     const gchar *ver_str;
-    guint major, minor, micro;
+    guint major, minor, micro, nano;
+    gint nscan;
 
     ver_str = gst_plugin_get_version (plugin);
     g_return_val_if_fail (ver_str != NULL, FALSE);
 
-    if (sscanf (ver_str, "%u.%u.%u", &major, &minor, &micro) == 3) {
+    nscan = sscanf (ver_str, "%u.%u.%u.%u", &major, &minor, &micro, &nano);
+    GST_DEBUG ("version string '%s' parsed to %d values", ver_str, nscan);
+
+    if (nscan >= 3) {
       if (major > min_major)
         ret = TRUE;
       else if (major < min_major)
@@ -307,6 +347,10 @@ gst_plugin_feature_check_version (GstPluginFeature * feature,
         ret = FALSE;
       else if (micro > min_micro)
         ret = TRUE;
+      /* micro is 1 smaller but we have a nano version, this is the upcomming
+       * release of the requested version and we're ok then */
+      else if (nscan == 4 && nano > 0 && (micro + 1 == min_micro))
+        ret = TRUE;
       else
         ret = (micro == min_micro);
 
