@@ -254,6 +254,10 @@ struct _GstBaseTransformPrivate
   gboolean suggest_pending;
 
   gboolean reconfigure;
+
+  /* QoS stats */
+  guint64 processed;
+  guint64 dropped;
 };
 
 static GstElementClass *parent_class = NULL;
@@ -317,6 +321,8 @@ static GstFlowReturn gst_base_transform_chain (GstPad * pad,
     GstBuffer * buffer);
 static GstCaps *gst_base_transform_getcaps (GstPad * pad);
 static gboolean gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps);
+static gboolean gst_base_transform_acceptcaps_default (GstBaseTransform * trans,
+    GstPadDirection direction, GstCaps * caps);
 static gboolean gst_base_transform_setcaps (GstPad * pad, GstCaps * caps);
 static GstFlowReturn gst_base_transform_buffer_alloc (GstPad * pad,
     guint64 offset, guint size, GstCaps * caps, GstBuffer ** buf);
@@ -364,6 +370,8 @@ gst_base_transform_class_init (GstBaseTransformClass * klass)
   klass->passthrough_on_same_caps = FALSE;
   klass->event = GST_DEBUG_FUNCPTR (gst_base_transform_sink_eventfunc);
   klass->src_event = GST_DEBUG_FUNCPTR (gst_base_transform_src_eventfunc);
+  klass->accept_caps =
+      GST_DEBUG_FUNCPTR (gst_base_transform_acceptcaps_default);
 }
 
 static void
@@ -433,6 +441,9 @@ gst_base_transform_init (GstBaseTransform * trans,
       trans->passthrough = TRUE;
     }
   }
+
+  trans->priv->processed = 0;
+  trans->priv->dropped = 0;
 }
 
 /* given @caps on the src or sink pad (given by @direction)
@@ -537,6 +548,11 @@ gst_base_transform_transform_size (GstBaseTransform * trans,
     /* if there is a custom transform function, use this */
     ret = klass->transform_size (trans, direction, caps, size, othercaps,
         othersize);
+  } else if (klass->get_unit_size == NULL) {
+    /* if there is no transform_size and no unit_size, it means the
+     * element does not modify the size of a buffer */
+    *othersize = size;
+    ret = TRUE;
   } else {
     /* there is no transform_size function, we have to use the unit_size
      * functions. This method assumes there is a fixed unit_size associated with
@@ -693,6 +709,11 @@ gst_base_transform_configure_caps (GstBaseTransform * trans, GstCaps * in,
     ret = klass->set_caps (trans, in, out);
   }
 
+  GST_OBJECT_LOCK (trans);
+  /* make sure we reevaluate how the buffer_alloc works wrt to proxy allocating
+   * the buffer. */
+  trans->priv->suggest_pending = TRUE;
+  GST_OBJECT_UNLOCK (trans);
   trans->negotiated = ret;
 
   return ret;
@@ -873,8 +894,6 @@ gst_base_transform_find_transform (GstBaseTransform * trans, GstPad * pad,
   /* third attempt at fixation, call the fixate vmethod and
    * ultimately call the pad fixate function. */
   if (!is_fixed) {
-    GstCaps *temp;
-
     GST_DEBUG_OBJECT (trans,
         "trying to fixate %" GST_PTR_FORMAT " on pad %s:%s",
         othercaps, GST_DEBUG_PAD_NAME (otherpad));
@@ -884,9 +903,7 @@ gst_base_transform_find_transform (GstBaseTransform * trans, GstPad * pad,
 
     /* FIXME: when fixating using the vmethod, it might make sense to fixate
      * each of the caps; but Wim doesn't see a use case for that yet */
-    temp = gst_caps_copy_nth (othercaps, 0);
-    gst_caps_unref (othercaps);
-    othercaps = temp;
+    gst_caps_truncate (othercaps);
     peer_checked = FALSE;
 
     if (klass->fixate_caps) {
@@ -976,17 +993,15 @@ error_cleanup:
 }
 
 static gboolean
-gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps)
+gst_base_transform_acceptcaps_default (GstBaseTransform * trans,
+    GstPadDirection direction, GstCaps * caps)
 {
-  GstBaseTransform *trans;
 #if 0
   GstPad *otherpad;
   GstCaps *othercaps = NULL;
 #endif
   gboolean ret = TRUE;
 
-  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
-
 #if 0
   otherpad = (pad == trans->srcpad) ? trans->sinkpad : trans->srcpad;
 
@@ -997,16 +1012,20 @@ gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps)
   {
     GstCaps *allowed;
 
-    GST_DEBUG_OBJECT (pad, "non fixed accept caps %" GST_PTR_FORMAT, caps);
+    GST_DEBUG_OBJECT (trans, "non fixed accept caps %" GST_PTR_FORMAT, caps);
 
     /* get all the formats we can handle on this pad */
-    allowed = gst_pad_get_caps_reffed (pad);
+    if (direction == GST_PAD_SRC)
+      allowed = gst_pad_get_caps_reffed (trans->srcpad);
+    else
+      allowed = gst_pad_get_caps_reffed (trans->sinkpad);
+
     if (!allowed) {
-      GST_DEBUG_OBJECT (pad, "gst_pad_get_caps() failed");
+      GST_DEBUG_OBJECT (trans, "gst_pad_get_caps() failed");
       goto no_transform_possible;
     }
 
-    GST_DEBUG_OBJECT (pad, "allowed caps %" GST_PTR_FORMAT, allowed);
+    GST_DEBUG_OBJECT (trans, "allowed caps %" GST_PTR_FORMAT, allowed);
 
     /* intersect with the requested format */
     ret = gst_caps_can_intersect (allowed, caps);
@@ -1035,7 +1054,6 @@ done:
   if (othercaps)
     gst_caps_unref (othercaps);
 #endif
-  gst_object_unref (trans);
 
   return ret;
 
@@ -1050,6 +1068,24 @@ no_transform_possible:
   }
 }
 
+static gboolean
+gst_base_transform_acceptcaps (GstPad * pad, GstCaps * caps)
+{
+  gboolean ret = TRUE;
+  GstBaseTransform *trans;
+  GstBaseTransformClass *bclass;
+
+  trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
+  bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
+
+  if (bclass->accept_caps)
+    ret = bclass->accept_caps (trans, GST_PAD_DIRECTION (pad), caps);
+
+  gst_object_unref (trans);
+
+  return ret;
+}
+
 /* called when new caps arrive on the sink or source pad,
  * We try to find the best caps for the other side using our _find_transform()
  * function. If there are caps, we configure the transform for this new
@@ -1517,7 +1553,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   GstBaseTransformPrivate *priv;
   GstFlowReturn res;
   gboolean proxy, suggest, same_caps;
-  GstCaps *sink_suggest;
+  GstCaps *sink_suggest = NULL;
   guint size_suggest;
 
   trans = GST_BASE_TRANSFORM (gst_pad_get_parent (pad));
@@ -1542,7 +1578,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
   if (same_caps) {
     /* we have seen this before, see below if we need to proxy */
     GST_DEBUG_OBJECT (trans, "have old caps %p, size %u", caps, size);
-    sink_suggest = caps;
+    gst_caps_replace (&sink_suggest, caps);
     size_suggest = size;
     suggest = FALSE;
   } else {
@@ -1550,8 +1586,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
 
     /* if we have a suggestion, pretend we got these as input */
     GST_OBJECT_LOCK (pad);
-    if ((priv->size_suggest && priv->sink_suggest
-            && !gst_caps_is_equal (caps, priv->sink_suggest))) {
+    if ((priv->sink_suggest && !gst_caps_is_equal (caps, priv->sink_suggest))) {
       sink_suggest = gst_caps_ref (priv->sink_suggest);
       size_suggest = priv->size_suggest;
       GST_DEBUG_OBJECT (trans, "have suggestion %p %" GST_PTR_FORMAT " size %u",
@@ -1562,7 +1597,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
     } else {
       GST_DEBUG_OBJECT (trans, "using caps %p %" GST_PTR_FORMAT " size %u",
           caps, caps, size);
-      sink_suggest = caps;
+      gst_caps_replace (&sink_suggest, caps);
       size_suggest = size;
       suggest = FALSE;
     }
@@ -1571,10 +1606,48 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
 
     /* check if we actually handle this format on the sinkpad */
     if (sink_suggest) {
-      const GstCaps *templ = gst_pad_get_pad_template_caps (pad);
+      const GstCaps *templ;
 
-      if (!gst_caps_can_intersect (sink_suggest, templ))
-        goto not_supported;
+      if (!gst_caps_is_fixed (sink_suggest)) {
+        GstCaps *peercaps;
+
+        GST_DEBUG_OBJECT (trans, "Suggested caps is not fixed: %"
+            GST_PTR_FORMAT, sink_suggest);
+
+        peercaps =
+            gst_pad_peer_get_caps_reffed (GST_BASE_TRANSFORM_SINK_PAD (trans));
+        /* try fixating by intersecting with peer caps */
+        if (peercaps) {
+          GstCaps *intersect;
+
+          intersect = gst_caps_intersect (peercaps, sink_suggest);
+          gst_caps_unref (peercaps);
+          gst_caps_unref (sink_suggest);
+          sink_suggest = intersect;
+        }
+
+        if (gst_caps_is_empty (sink_suggest))
+          goto not_supported;
+
+        /* be safe and call default fixate */
+        sink_suggest = gst_caps_make_writable (sink_suggest);
+        gst_pad_fixate_caps (GST_BASE_TRANSFORM_SINK_PAD (trans), sink_suggest);
+
+        if (!gst_caps_is_fixed (sink_suggest)) {
+          gst_caps_unref (sink_suggest);
+          sink_suggest = NULL;
+        }
+
+        GST_DEBUG_OBJECT (trans, "Caps fixated to now: %" GST_PTR_FORMAT,
+            sink_suggest);
+      }
+
+      if (sink_suggest) {
+        templ = gst_pad_get_pad_template_caps (pad);
+
+        if (!gst_caps_can_intersect (sink_suggest, templ))
+          goto not_supported;
+      }
     }
 
     /* find the best format for the other side here we decide if we will proxy
@@ -1665,9 +1738,12 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
         "doing suggestion of size %u, caps %p %" GST_PTR_FORMAT, size_suggest,
         sink_suggest, sink_suggest);
     GST_BUFFER_CAPS (*buf) = sink_suggest;
+    sink_suggest = NULL;
   }
 
   gst_object_unref (trans);
+  if (sink_suggest)
+    gst_caps_unref (sink_suggest);
 
   return res;
 
@@ -1675,7 +1751,7 @@ gst_base_transform_buffer_alloc (GstPad * pad, guint64 offset, guint size,
 alloc_failed:
   {
     GST_DEBUG_OBJECT (trans, "pad alloc failed: %s", gst_flow_get_name (res));
-    if (suggest)
+    if (sink_suggest)
       gst_caps_unref (sink_suggest);
     gst_object_unref (trans);
     return res;
@@ -1683,7 +1759,7 @@ alloc_failed:
 not_supported:
   {
     GST_DEBUG_OBJECT (trans, "pad alloc with unsupported caps");
-    if (suggest)
+    if (sink_suggest)
       gst_caps_unref (sink_suggest);
     gst_object_unref (trans);
     return GST_FLOW_NOT_NEGOTIATED;
@@ -1728,6 +1804,8 @@ gst_base_transform_sink_eventfunc (GstBaseTransform * trans, GstEvent * event)
       trans->priv->proportion = 1.0;
       trans->priv->earliest_time = -1;
       trans->priv->discont = FALSE;
+      trans->priv->processed = 0;
+      trans->priv->dropped = 0;
       GST_OBJECT_UNLOCK (trans);
       /* we need new segment info after the flush. */
       trans->have_newsegment = FALSE;
@@ -1835,7 +1913,8 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
   GstBaseTransformClass *bclass;
   GstFlowReturn ret = GST_FLOW_OK;
   gboolean want_in_place, reconfigure;
-  GstClockTime qostime;
+  GstClockTime running_time;
+  GstClockTime timestamp;
   GstCaps *incaps;
 
   bclass = GST_BASE_TRANSFORM_GET_CLASS (trans);
@@ -1882,27 +1961,52 @@ gst_base_transform_handle_buffer (GstBaseTransform * trans, GstBuffer * inbuf,
     goto no_qos;
 
   /* QOS is done on the running time of the buffer, get it now */
-  qostime = gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
-      GST_BUFFER_TIMESTAMP (inbuf));
+  timestamp = GST_BUFFER_TIMESTAMP (inbuf);
+  running_time = gst_segment_to_running_time (&trans->segment, GST_FORMAT_TIME,
+      timestamp);
 
-  if (qostime != -1) {
+  if (running_time != -1) {
     gboolean need_skip;
     GstClockTime earliest_time;
+    gdouble proportion;
 
     /* lock for getting the QoS parameters that are set (in a different thread)
      * with the QOS events */
     GST_OBJECT_LOCK (trans);
     earliest_time = trans->priv->earliest_time;
+    proportion = trans->priv->proportion;
     /* check for QoS, don't perform conversion for buffers
      * that are known to be late. */
     need_skip = trans->priv->qos_enabled &&
-        earliest_time != -1 && qostime != -1 && qostime <= earliest_time;
+        earliest_time != -1 && running_time <= earliest_time;
     GST_OBJECT_UNLOCK (trans);
 
     if (need_skip) {
+      GstMessage *qos_msg;
+      GstClockTime duration;
+      guint64 stream_time;
+      gint64 jitter;
+
       GST_CAT_DEBUG_OBJECT (GST_CAT_QOS, trans, "skipping transform: qostime %"
           GST_TIME_FORMAT " <= %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (qostime), GST_TIME_ARGS (earliest_time));
+          GST_TIME_ARGS (running_time), GST_TIME_ARGS (earliest_time));
+
+      trans->priv->dropped++;
+
+      duration = GST_BUFFER_DURATION (inbuf);
+      stream_time =
+          gst_segment_to_stream_time (&trans->segment, GST_FORMAT_TIME,
+          timestamp);
+      jitter = GST_CLOCK_DIFF (running_time, earliest_time);
+
+      qos_msg =
+          gst_message_new_qos (GST_OBJECT_CAST (trans), FALSE, running_time,
+          stream_time, timestamp, duration);
+      gst_message_set_qos_values (qos_msg, jitter, proportion, 1000000);
+      gst_message_set_qos_stats (qos_msg, GST_FORMAT_BUFFERS,
+          trans->priv->processed, trans->priv->dropped);
+      gst_element_post_message (GST_ELEMENT_CAST (trans), qos_msg);
+
       /* mark discont for next buffer */
       trans->priv->discont = TRUE;
       goto skip;
@@ -2040,16 +2144,20 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
   GstBaseTransformClass *klass;
   GstFlowReturn ret;
   GstClockTime last_stop = GST_CLOCK_TIME_NONE;
+  GstClockTime timestamp, duration;
   GstBuffer *outbuf = NULL;
 
   trans = GST_BASE_TRANSFORM (GST_OBJECT_PARENT (pad));
 
+  timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  duration = GST_BUFFER_DURATION (buffer);
+
   /* calculate end position of the incoming buffer */
-  if (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE) {
-    if (GST_BUFFER_DURATION (buffer) != GST_CLOCK_TIME_NONE)
-      last_stop = GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer);
+  if (timestamp != GST_CLOCK_TIME_NONE) {
+    if (duration != GST_CLOCK_TIME_NONE)
+      last_stop = timestamp + duration;
     else
-      last_stop = GST_BUFFER_TIMESTAMP (buffer);
+      last_stop = timestamp;
   }
 
   klass = GST_BASE_TRANSFORM_GET_CLASS (trans);
@@ -2078,9 +2186,11 @@ gst_base_transform_chain (GstPad * pad, GstBuffer * buffer)
         }
         trans->priv->discont = FALSE;
       }
+      trans->priv->processed++;
       ret = gst_pad_push (trans->srcpad, outbuf);
-    } else
+    } else {
       gst_buffer_unref (outbuf);
+    }
   }
 
   /* convert internal flow to OK and mark discont for the next buffer. */
@@ -2157,6 +2267,8 @@ gst_base_transform_activate (GstBaseTransform * trans, gboolean active)
     trans->priv->earliest_time = -1;
     trans->priv->discont = FALSE;
     gst_caps_replace (&trans->priv->sink_suggest, NULL);
+    trans->priv->processed = 0;
+    trans->priv->dropped = 0;
 
     GST_OBJECT_UNLOCK (trans);
   } else {
