@@ -82,6 +82,35 @@ static void gst_check_log_critical_func
     _gst_check_raised_warning = TRUE;
 }
 
+static gint
+sort_plugins (GstPlugin * a, GstPlugin * b)
+{
+  int ret;
+
+  ret = strcmp (gst_plugin_get_source (a), gst_plugin_get_source (b));
+  if (ret == 0)
+    ret = strcmp (gst_plugin_get_name (a), gst_plugin_get_name (b));
+  return ret;
+}
+
+static void
+print_plugins (void)
+{
+  GList *plugins, *l;
+
+  plugins = gst_default_registry_get_plugin_list ();
+  plugins = g_list_sort (plugins, (GCompareFunc) sort_plugins);
+  for (l = plugins; l != NULL; l = l->next) {
+    GstPlugin *plugin = GST_PLUGIN (l->data);
+
+    if (strcmp (gst_plugin_get_source (plugin), "BLACKLIST") != 0) {
+      GST_LOG ("%20s@%s", gst_plugin_get_name (plugin),
+          GST_STR_NULL (gst_plugin_get_filename (plugin)));
+    }
+  }
+  gst_plugin_list_free (plugins);
+}
+
 /* initialize GStreamer testing */
 void
 gst_check_init (int *argc, char **argv[])
@@ -104,6 +133,8 @@ gst_check_init (int *argc, char **argv[])
   g_log_set_handler ("Gst-Phonon", G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING,
       gst_check_log_critical_func, NULL);
 
+  print_plugins ();
+
   check_cond = g_cond_new ();
   check_mutex = g_mutex_new ();
 }
@@ -182,7 +213,7 @@ gst_check_setup_src_pad (GstElement * element,
 
 GstPad *
 gst_check_setup_src_pad_by_name (GstElement * element,
-    GstStaticPadTemplate * template, gchar * name)
+    GstStaticPadTemplate * template, const gchar * name)
 {
   GstPad *srcpad, *sinkpad;
 
@@ -207,7 +238,7 @@ gst_check_setup_src_pad_by_name (GstElement * element,
 }
 
 void
-gst_check_teardown_pad_by_name (GstElement * element, gchar * name)
+gst_check_teardown_pad_by_name (GstElement * element, const gchar * name)
 {
   GstPad *pad_peer, *pad_element;
 
@@ -261,7 +292,7 @@ gst_check_setup_sink_pad (GstElement * element, GstStaticPadTemplate * template,
 
 GstPad *
 gst_check_setup_sink_pad_by_name (GstElement * element,
-    GstStaticPadTemplate * template, gchar * name)
+    GstStaticPadTemplate * template, const gchar * name)
 {
   GstPad *srcpad, *sinkpad;
 
@@ -336,8 +367,8 @@ gst_check_caps_equal (GstCaps * caps1, GstCaps * caps2)
 }
 
 /**
- * gst_check_element_push_buffer:
- * @element: name of the element that needs to be created
+ * gst_check_element_push_buffer_list:
+ * @element_name: name of the element that needs to be created
  * @buffer_in: a list of buffers that needs to be puched to the element
  * @buffer_out: a list of buffers that we expect from the element
  * @last_flow_return: the last buffer push needs to give this GstFlowReturn
@@ -353,6 +384,7 @@ gst_check_caps_equal (GstCaps * caps1, GstCaps * caps2)
  * 
  * Since: 0.10.18
  */
+/* FIXME 0.11: rename this function now that there's GstBufferList? */
 void
 gst_check_element_push_buffer_list (const gchar * element_name,
     GList * buffer_in, GList * buffer_out, GstFlowReturn last_flow_return)
@@ -442,6 +474,13 @@ gst_check_element_push_buffer_list (const gchar * element_name,
     GstBuffer *new = GST_BUFFER (buffers->data);
     GstBuffer *orig = GST_BUFFER (buffer_out->data);
 
+    GST_LOG ("orig buffer: size %u, caps %" GST_PTR_FORMAT,
+        GST_BUFFER_SIZE (orig), GST_BUFFER_CAPS (orig));
+    GST_LOG ("new  buffer: size %u, caps %" GST_PTR_FORMAT,
+        GST_BUFFER_SIZE (new), GST_BUFFER_CAPS (new));
+    GST_MEMDUMP ("orig buffer", GST_BUFFER_DATA (orig), GST_BUFFER_SIZE (orig));
+    GST_MEMDUMP ("new  buffer", GST_BUFFER_DATA (new), GST_BUFFER_SIZE (new));
+
     /* remove the buffers */
     buffers = g_list_remove (buffers, new);
     buffer_out = g_list_remove (buffer_out, orig);
@@ -567,7 +606,7 @@ _gst_check_run_test_func (const gchar * func_name)
   /* only run specified functions */
   funcs = g_strsplit (gst_checks, ",", -1);
   for (f = funcs; f != NULL && *f != NULL; ++f) {
-    if (strcmp (*f, func_name) == 0) {
+    if (g_pattern_match_simple (*f, func_name)) {
       res = TRUE;
       break;
     }
